package jumpstart.min.business.commons.exception;

import java.io.Serializable;

import jumpstart.min.business.domain.base.BaseEntity;
import jumpstart.min.util.ClassUtil;
import jumpstart.min.util.MessageUtil;

@SuppressWarnings("serial")
public class DuplicatePrimaryKeyException extends BusinessException {
	static public final int INFORMATIONLEVEL_ENTITY_ID = 1;
	static public final int INFORMATIONLEVEL_ID = 2;
	static public final int INFORMATIONLEVEL_NONE = 3;

	private int _informationLevel = INFORMATIONLEVEL_ENTITY_ID;
	private String _entityLabelMessageId;
	private Serializable _id;

	public DuplicatePrimaryKeyException(BaseEntity entity, Serializable id) {

		// Don't convert the message ids to messages yet because we're in the
		// server's locale, not the user's.

		super("DuplicatePrimaryKeyException");
		_informationLevel = INFORMATIONLEVEL_ENTITY_ID;
		_entityLabelMessageId = ClassUtil.extractUnqualifiedName(entity);
		_id = id;
	}

	public DuplicatePrimaryKeyException(String entityLabelMessageId, Serializable id) {

		// Don't convert the message ids to messages yet because we're in the
		// server's locale, not the user's.

		super("DuplicatePrimaryKeyException");
		_informationLevel = INFORMATIONLEVEL_ENTITY_ID;
		_entityLabelMessageId = entityLabelMessageId;
		_id = id;
	}

	public DuplicatePrimaryKeyException(Serializable id) {

		// Don't convert the message ids to messages yet because we're in the
		// server's locale, not the user's.

		super("DuplicatePrimaryKeyException_2");
		_informationLevel = INFORMATIONLEVEL_ID;
		_id = id;
	}

	public DuplicatePrimaryKeyException() {

		// Don't convert the message ids to messages yet because we're in the
		// server's locale, not the user's.

		super("DuplicatePrimaryKeyException_3");
		_informationLevel = INFORMATIONLEVEL_NONE;
	}

	@Override
	public String getMessage() {

		// We deferred converting the message ids to messages until now, when we
		// are more likely to be in the user's locale.

		Object[] msgArgs;
		if (_informationLevel == INFORMATIONLEVEL_ENTITY_ID) {
			msgArgs = new Object[] { MessageUtil.toText(_entityLabelMessageId), _id };
		}
		else if (_informationLevel == INFORMATIONLEVEL_ID) {
			msgArgs = new Object[] { _id };
		}
		else if (_informationLevel == INFORMATIONLEVEL_NONE) {
			msgArgs = new Object[] { };
		}
		else {
			throw new IllegalStateException("_informationLevel = " + _informationLevel);
		}

		String msg = MessageUtil.toText(_messageId, msgArgs);
		return msg;
	}

	public String getEntityLabelMessageId() {
		return _entityLabelMessageId;
	}

	public Serializable getId() {
		return _id;
	}

	public int getInformationLevel() {
		return _informationLevel;
	}
}
