package jumpstart.min.util;

abstract public class UserFriendlyException extends Exception {
	protected String _messageId = "";
	protected Object[] _messageArgs = null;
	protected Exception _exception = null;
	
	public UserFriendlyException(String messageId) {
		this(messageId, null, null);
	}
	
	public UserFriendlyException(String messageId, Exception exception) {
		this(messageId, exception, null);
	}
	
	public UserFriendlyException(String messageId, Exception exception, Object[] messageArgs) {
		_messageId = messageId;
		_messageArgs = messageArgs;
		_exception = exception;
	}
	
	public UserFriendlyException(String messageId, Exception exception, Object messageArg) {
		this(messageId, exception, new Object[] { messageArg });
	}
	
	public UserFriendlyException(String messageId, Exception exception, Object messageArg1, Object messageArg2) {
		this(messageId, exception, new Object[] { messageArg1, messageArg2 });
	}
	
	public UserFriendlyException(String messageId, Object[] messageArgs) {
		this(messageId, null, messageArgs);
	}
	
	public UserFriendlyException(String messageId, Object messageArg) {
		this(messageId, null, new Object[] { messageArg });
	}
	
	public UserFriendlyException(String messageId, Object messageArg1, Object messageArg2) {
		this(messageId, null, new Object[] { messageArg1, messageArg2 });
	}
	
	public String getMessageId() {
			return _messageId;
	}
	
	public Object[] getMessageArgs() {
			return _messageArgs;
	}
	
	public String getMessage() {
		if (_messageArgs == null) {
			return MessageUtil.toText(_messageId);
		}
		else {
			return MessageUtil.toText(_messageId, _messageArgs);
		}
	}
	
	public String toString() {
		String str = getMessage();
		if (_exception != null) {
			str = str + ": " + _exception.toString();
		}
		return str;
	}
}
