package jumpstart.min.business;

import java.io.IOException;
import java.util.List;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.naming.NoInitialContextException;

import jumpstart.min.business.commons.exception.BusinessException;
import jumpstart.min.business.commons.exception.ServiceException;
import jumpstart.min.business.commons.interpreter.BizServicesExceptionInterpreter;
import jumpstart.min.business.commons.query.SearchOptions;
import jumpstart.min.business.domain.security.User;
import jumpstart.min.business.domain.security.iface.ISecurityFinderSvcRemote;
import jumpstart.min.business.domain.security.iface.ISecurityManagerSvcRemote;
import jumpstart.min.business.domain.security.iface.UserSearchFields;
import jumpstart.min.util.ExceptionFormatter;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class AdHocTester {
	static private final Log LOG = LogFactory.getLog(AdHocTester.class);

	static private ISecurityFinderSvcRemote _securityFinderSvc;
	@SuppressWarnings("unused")
	static private ISecurityManagerSvcRemote _securityManagerSvc;

	static private BizServicesExceptionInterpreter _interpreter = new BizServicesExceptionInterpreter();

	static public void main(String[] args) {

		try {
			locateServices();
			AdHocTester tester = new AdHocTester();
			tester.run("admin", "admin");
		}
		catch (Exception e) {
			ServiceException s = _interpreter.interpret(e);
			LOG.error(ExceptionFormatter.toAString(s));
		}
	}

	static public void locateServices() {
		InitialContext initialContext = null;

		try {
			initialContext = new InitialContext();
			System.out.println("InitialContext environment = " + initialContext.getEnvironment());
			_securityFinderSvc = (ISecurityFinderSvcRemote) initialContext.lookup("jumpstart-min/SecurityFinderSvc/remote");
			_securityManagerSvc = (ISecurityManagerSvcRemote) initialContext.lookup("jumpstart-min/SecurityManagerSvc/remote");
		}
		catch (NoInitialContextException e) {
			e.printStackTrace();
			throw new RuntimeException("Could not locate services.  Is jndi.properties missing from the testing classpath?", e);
		}
		catch (NamingException e) {
			e.printStackTrace();
			throw new RuntimeException(
					"Could not locate services.  Is JBoss server started?  Is jbossall-client.jar missing from the testing classpath?",
					e);
		}

	}
	public void run(String adminUserName, String adminPassword) throws IOException {
		// Put your adhoc tests here
		
		try {
			User u = _securityFinderSvc.findUser((long) 2);
			System.out.println(u);
			
			try {
				u = _securityFinderSvc.findUser((long) 22222);
				System.out.println(u);
			} catch (Exception e) {
				System.out.println("Good. " + e);
			}
			
			List<User> l = _securityFinderSvc.findUsers();
			System.out.println(l);
			
			UserSearchFields searchFields = new UserSearchFields();
			SearchOptions searchOptions = new SearchOptions();
			l = _securityFinderSvc.findUsers(searchFields, searchOptions);
			System.out.println("Size: " + l.size());
			System.out.println(l);
			
		} catch (BusinessException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
}
