package jumpstart.min.web.pages.security;

import jumpstart.min.business.commons.exception.ServiceException;
import jumpstart.min.business.domain.security.User;
import jumpstart.min.business.domain.security.iface.ISecurityManagerSvcLocal;
import jumpstart.min.web.pages.base.ProtectedBasePage;

import org.apache.tapestry.annotations.Persist;
import org.apache.tapestry.form.IPropertySelectionModel;
import org.apache.tapestry.form.StringPropertySelectionModel;

public abstract class UserCreatePage extends ProtectedBasePage {
	static public final String PAGE_NAME = "pages/security/UserCreatePage";

	@Persist("client")
	public abstract User getUser();
	public abstract void setUser(User value);
	
	public abstract String getPassword();
	public abstract String getConfirmPassword();

	@Persist("session")
	// Cannot use client persistence because not serializable
	public abstract IPropertySelectionModel getSalutationsModel();
	public abstract void setSalutationsModel(IPropertySelectionModel value);

	public void activate() throws ServiceException {
		User u = new User();
		u.setActive(true);
		setUser(u);

		// Put a selection list of salutations onto the page

		setSalutationsModel(new StringPropertySelectionModel(User.salutations));

		getRequestCycle().activate(this);
	}

	public void doSave() {

		if (!hasErrors()) {
			validateSave();

			if (!hasErrors()) {
				try {
					ISecurityManagerSvcLocal manager = getBizServicesLocator().getSecurityManagerSvcLocal();

					// Create the User

					User user = getUser();
					String password = getPassword();
					manager.createUser(user, password);

					UserSearchPage nextPage = (UserSearchPage) getRequestCycle().getPage(UserSearchPage.PAGE_NAME);
					nextPage.activateAgain();
				}
				catch (Exception e) {
					handleBizServicesExceptionForCreate(e);
				}
			}
		}

	}

	private void validateSave() {

		if (!getPassword().equals(getConfirmPassword())) {
			recordError(getMessages().getMessage("User_confirmpassword_does_not_match"), "password");
		}
	}

	public void doCancel() {
		UserSearchPage nextPage = (UserSearchPage) getRequestCycle().getPage(UserSearchPage.PAGE_NAME);
		nextPage.activateAgain();
	}

}
