package jumpstart.min.web.pages.security;

import java.util.Arrays;
import java.util.List;

import jumpstart.min.business.commons.exception.ServiceException;
import jumpstart.min.business.commons.query.SearchOptions;
import jumpstart.min.business.domain.security.User;
import jumpstart.min.business.domain.security.iface.ISecurityFinderSvcLocal;
import jumpstart.min.business.domain.security.iface.ISecurityManagerSvcLocal;
import jumpstart.min.business.domain.security.iface.UserSearchFields;
import jumpstart.min.web.commons.form.ObjectPropertySelectionModel;
import jumpstart.min.web.pages.base.ProtectedBasePage;

import org.apache.tapestry.annotations.Bean;
import org.apache.tapestry.annotations.Persist;
import org.apache.tapestry.bean.EvenOdd;
import org.apache.tapestry.form.IPropertySelectionModel;
import org.apache.tapestry.form.StringPropertySelectionModel;

/**
 * Page class for the application's List${entityBean.Name} page.
 * 
 * @author Geoff Callender JumpStart
 * @version $Revision: 1.10 $, $Date: 2006/02/15 10:43:06 $
 */
public abstract class UserSearchPage extends ProtectedBasePage {
	static public final String PAGE_NAME = "pages/security/UserSearchPage";

	@Persist("client")
	public abstract UserSearchFields getSearch();
	public abstract void setSearch(UserSearchFields value);

	@Persist("session")
	// Keep in session for activateAgain()
	public abstract UserSearchFields getLastSearch();
	public abstract void setLastSearch(UserSearchFields value);

	@Persist("session")
	// Cannot use client persistence because not serializable
	public abstract IPropertySelectionModel getSalutationsModel();
	public abstract void setSalutationsModel(IPropertySelectionModel value);

	@Persist("session")
	// Cannot use client persistence because not serializable
	public abstract IPropertySelectionModel getActivesModel();
	public abstract void setActivesModel(IPropertySelectionModel value);

	@Persist("session")
	// Probably best not to use client persistence, although it does encode the
	// data, because User inludes password
	public abstract List<User> getUsers();
	public abstract void setUsers(List<User> value);

	public abstract User getUser();
	public abstract void setUser(User value);

	@Bean
	public abstract EvenOdd getEvenOdd();

	public void activate() {
		doResetSearch();

		getRequestCycle().activate(this);
	}

	public void activateAgain() {

		if (getLastSearch() == null) {
			// Don't call doResetSearch() because it would
			// unnecessarily refresh the selection lists
			setLastSearch(null);
			setSearch(new UserSearchFields());
		}
		else {
			UserSearchFields searchFields = getLastSearch(); 
			doSearch(searchFields);
			setSearch(searchFields);
			setLastSearch(searchFields);
		}

		getRequestCycle().activate(this);
	}

	public void doResetSearch() {
		setSearch(new UserSearchFields());
		setLastSearch(null);
		setUsers(null);
		refreshSelectionLists();
	}

	public void refreshSelectionLists() {

		// Put a selection list of salutations onto the page

		setSalutationsModel(new StringPropertySelectionModel(User.salutations));
		
		setActivesModel(new ObjectPropertySelectionModel(Arrays.asList(UserSearchFields.actives), Boolean.class, "toString", null, true, ""));
	}

	public void doSearch(UserSearchFields searchFields) {

		if (!hasErrors()) {
			ISecurityFinderSvcLocal finder = getBizServicesLocator().getSecurityFinderSvcLocal();

			SearchOptions searchOptions = new SearchOptions();
			List<User> l = finder.findUsers(searchFields, searchOptions);
			
			setUsers(l);

			setLastSearch(searchFields);
		}
	}

	public void doNew() {

		try {
			UserCreatePage nextPage = (UserCreatePage) getRequestCycle().getPage(UserCreatePage.PAGE_NAME);
			nextPage.activate();
		}
		catch (ServiceException e) {
			recordError(e.toString());
		}

	}

	public void doView(Long id) {

		try {
			UserViewPage nextPage = (UserViewPage) getRequestCycle().getPage(UserViewPage.PAGE_NAME);
			nextPage.activate(id);
		}
		catch (ServiceException e) {
			recordError(e.toString());
		}

	}

	public void doEdit(Long id) {

		try {
			UserEditPage nextPage = (UserEditPage) getRequestCycle().getPage(UserEditPage.PAGE_NAME);
			nextPage.activate(id);
		}
		catch (ServiceException e) {
			recordError(e.toString());
		}

	}

	public void doDelete(Long id) {

		if (!hasErrors()) {

			// Find the User in the list by id

			User user = null;
			for (User user1 : getUsers()) {
				if (user1.getId().equals(id)) {
					user = user1;
					break;
				}
			}

			try {
				// Delete the User

				ISecurityManagerSvcLocal manager = getBizServicesLocator().getSecurityManagerSvcLocal();
				manager.deleteUser(user);

				// Update the users list on screen

				List<User> users = getUsers();
				users.remove(user);
				setUsers(users);
			}
			catch (Exception e) {
				handleBizServicesExceptionForDelete(e);
			}
		}

	}

}
