package jumpstart.min.web.state;

import javax.naming.InitialContext;
import javax.naming.NamingException;

import jumpstart.min.business.domain.security.iface.ISecurityFinderSvcLocal;
import jumpstart.min.business.domain.security.iface.ISecurityManagerSvcLocal;

/**
 * BizServicesLocator is used to centralize all JNDI lookups.
 * It is an Application-scoped State Object configured in hivemodule.xml.
 */

public class BizServicesLocator {

	private ISecurityFinderSvcLocal _securityFinderSvcLocal;
	private ISecurityManagerSvcLocal _securityManagerSvcLocal;

	private InitialContext _initialContext;
	private boolean _servicesLocated = false;

	public BizServicesLocator() {
	}

	protected synchronized void locateServices() {
		if (!_servicesLocated) {
			try {
				_initialContext = new InitialContext();
				
				_securityFinderSvcLocal = (ISecurityFinderSvcLocal) _initialContext.lookup("jumpstart-min/SecurityFinderSvc/local");
				_securityManagerSvcLocal = (ISecurityManagerSvcLocal) _initialContext.lookup("jumpstart-min/SecurityManagerSvc/local");

				_servicesLocated = true;
			}
			catch (NamingException e) {
				_servicesLocated = false;
				clear();
				e.printStackTrace();
				throw new RuntimeException("JNDI lookup failed.  Is JBoss server not started?  Is jbossall-client.jar missing from classpath?", e);
			}
		}
	}

	public ISecurityFinderSvcLocal getSecurityFinderSvcLocal() {
		locateServices();
		return _securityFinderSvcLocal;
	}

	public ISecurityManagerSvcLocal getSecurityManagerSvcLocal() {
		locateServices();
		return _securityManagerSvcLocal;
	}

	/**
	 * Invoked after any kind of naming or remote exception. Cached naming contexts and interfaces
	 * are discarded.
	 */

	public synchronized void clear() {
		_initialContext = null;
		
		_securityFinderSvcLocal = null;
		_securityManagerSvcLocal = null;
	}

}
